﻿#region Copyright (c) 2013-04, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion


namespace Amarok.Agents
{
	/// <summary>
	/// This static class provides various extension methods for <see cref="IAgent"/>.
	/// </summary>
	public static class AgentExtensions
	{
		#region ToAgentReference

		/// <summary>
		/// Returns an agent reference that refers to the specified agent. If the supplied agent itself is an 
		/// agent reference then that agent reference is returned again; otherwise a new agent reference is 
		/// created.
		/// </summary>
		/// 
		/// <param name="agent">
		/// The agent to which the agent reference should refer. Null references are allowed.</param>
		/// 
		/// <returns>
		/// An agent reference that refers to the supplied agent, or null if a null reference was supplied.</returns>
		public static AgentReference ToAgentReference(this IAgent agent)
		{
			if (agent == null)
				return null;

			var agentReference = agent as AgentReference;

			if (agentReference != null)
				return agentReference;

			return new AgentReference(agent);
		}

		#endregion

	}
}
