﻿#region Copyright (c) 2013-04, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion


namespace Amarok.Agents
{
	/// <summary>
	/// This interface represents an agent, which is a combination of a dispatcher and a publisher.
	/// 
	/// A dispatcher is an object that is capable to forward (dispatch) messages to appropriate message handler. A 
	/// dispatcher provides rich queuing and scheduling mechanism that decouple caller from callee. Messages posted 
	/// to the dispatcher are first enqueued into an internal message queue and then later forwarded to their message 
	/// handlers.
	/// 
	/// A publisher is an object that is capable to send out (publish) messages to interested consumers (subscribers). 
	/// In general a publisher serves two sides. First, the publishing side, second the side that manages consumer 
	/// subscriptions. This interface represents only the subscription managing side.
	/// </summary>
	public interface IAgent :
		ICompletable,
		IDispatcher,
		IAdvertiser
	{
	}
}
