﻿#region Copyright (c) 2012-11, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion


namespace Amarok.Agents
{
	/// <summary>
	/// This enumeration defines the possible execution modes of message handlers. Execution modes affect the 
	/// scheduling of message handlers. They are the primary mechanism to synchronize data access and also to 
	/// synchronize operations that would impact each other.
	/// </summary>
	public enum ExecutionMode
	{
		/// <summary>
		/// Message handlers with an execution mode of 'Independent' can run independently from all other message 
		/// handlers within the same context, meaning that they can run simultaneously to message handlers with 
		/// execution modes of 'Independent', 'Exclusive' and 'Concurrent'.
		/// </summary>
		Independent,

		/// <summary>
		/// Message handlers with an execution mode of 'Concurrent' can only run when no other message handler with 
		/// execution mode 'Exclusive' runs. Other message handlers with execution modes 'Concurrent' or 'Independent' 
		/// can run simultaneously.
		/// 
		/// This execution mode should be applied to message handlers that query state or data (readers).
		/// </summary>
		Concurrent,

		/// <summary>
		/// Message handlers with an execution mode of 'Exclusive' can only run when no other message handler with 
		/// execution mode 'Exclusive' or 'Concurrent' runs. Message handlers with execution mode 'Independent' do 
		/// not prevent running exclusive message handlers.
		/// 
		/// This execution mode should be applied to message handlers that modify state or data (writers).
		/// </summary>
		Exclusive,

	}
}
