﻿#region Copyright (c) 2012-11, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;
using System.Reflection;


namespace Amarok.Agents
{
	/// <summary>
	/// This attribute marks a method (static or member method) as an operation message handler.
	/// </summary>
	[AttributeUsage(AttributeTargets.Method, AllowMultiple = false, Inherited = true)]
	public sealed class OperationHandlerAttribute : Attribute
	{
		#region ++ Public Static Interface ++

		/// <summary>
		/// Returns a boolean value indicating whether the specified method is marked as an operation 
		/// message handler.
		/// </summary>
		/// 
		/// <param name="methodInfo">
		/// The method which should be tested.</param>
		/// 
		/// <returns>
		/// True, if the supplied method is marked as an operation message handler; otherwise False.</returns>
		/// 
		/// <exception cref="ArgumentNullException">
		/// A null reference was passed to a method that did not accept it as a valid argument.</exception>
		public static Boolean IsDefined(MethodInfo methodInfo)
		{
			Verify.NotNull(methodInfo, "methodInfo");

			return Attribute.IsDefined(
				methodInfo,
				typeof(OperationHandlerAttribute),
				true);
		}

		#endregion

	}
}
