﻿#region Copyright (c) 2013-04, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;


namespace Amarok.Agents
{
	/// <summary>
	/// This interface represents a dispatcher, which is an object that is capable to forward (dispatch) messages 
	/// to appropriate message handler. In general this is done by invoking the message handler method and supplying 
	/// the message as an argument to the invoked method. In addition, a dispatcher provides rich queuing and 
	/// scheduling mechanism that decouple caller from callee. Messages posted to the dispatcher are first enqueued 
	/// into an internal queue and then later forwarded to their respective handlers.
	/// </summary>
	public interface IDispatcher
	{
		/// <summary>
		/// Posts the message to the dispatcher.
		/// 
		/// The method stores the supplied message in the dispatcher's input message queues of interested consumers and 
		/// returns immediately, meaning it only schedules the message for further processing, but the actual processing 
		/// is done later in other threads.
		/// 
		/// Messages that the dispatcher is unable to handle, i.e. because no corresponding message handler exists, 
		/// will be dropped silently.
		/// 
		/// If there is no interested consumer at the time of posting, then the message will be dropped silently.
		/// 
		/// Messages that are posted after completing the dispatcher will be dropped silently.
		/// </summary>
		/// 
		/// <param name="message">
		/// The message that should be posted.</param>
		/// 
		/// <returns>
		/// A boolean value indicating whether the message being posted has been scheduled for further processing, or 
		/// False, if the message has been filter, the dispatcher is completing or already completed, or no matching 
		/// message handler exists.
		/// </returns>
		/// 
		/// <exception cref="ArgumentNullException">
		/// A null reference was passed to a method that did not accept it as a valid argument.</exception>
		Boolean Post(Message message);

	}
}
