﻿#region Copyright (c) 2013-04, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;
using System.Reflection;
using System.Threading.Tasks;


namespace Amarok.Agents
{
	/// <summary>
	/// </summary>
	internal sealed class InformationHandlerInfo : HandlerInfo
	{
		#region ++ Public Interface ++

		/// <summary>
		/// </summary>
		public InformationHandlerInfo(MethodInfo methodInfo, ParameterInfo parameterInfo, Boolean isAsync)
			: base(methodInfo, parameterInfo, isAsync)
		{
		}

		#endregion

		#region ## Overridable Methods ##

		/// <summary>
		/// </summary>
		protected override IHandler OnCreateHandler(Object target)
		{
			if (this.IsAsync)
				return _CreateInformationHandler(_CreateDelegateToAsyncHandler(target));
			else
				return _CreateInformationHandler(_CreateDelegateToHandler(target));
		}

		#endregion

		#region Implementation

		private IHandler _CreateInformationHandler(Delegate callback)
		{
			var openType = typeof(InformationHandler<>);

			var closedType = openType.MakeGenericType(
				this.MessageType);

			return (IHandler)Activator.CreateInstance(
				closedType,
				callback);
		}

		private Delegate _CreateDelegateToHandler(Object target)
		{
			var openType = typeof(Action<>);

			var closedType = openType.MakeGenericType(
				this.MessageType);

			return Delegate.CreateDelegate(
				closedType,
				target,
				this.MethodInfo);
		}

		private Delegate _CreateDelegateToAsyncHandler(Object target)
		{
			var openType = typeof(Func<,>);

			var closedType = openType.MakeGenericType(
				this.MessageType,
				typeof(Task));

			return Delegate.CreateDelegate(
				closedType,
				target,
				this.MethodInfo);
		}

		#endregion

	}
}
