﻿#region Copyright (c) 2012-11, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;


namespace Amarok.Agents
{
	/// <summary>
	/// This type represents an Information Message. Information Messages are a special kind of Message intended 
	/// to notify about changes. This can be any kind of changes: for example, changes of state like 'measurement 
	/// started' or changes of data like 'user added'.
	/// 
	/// Types derived from this base class should follow a naming convention. An Information Message should be 
	/// named after the object that changed (what changed) followed by a verb that describes how it changed. Here 
	/// are a few examples: MeasurementStarted, UserAdded, ApplicationExitRequested, WindowClosed.
	/// 
	/// Information Messages should contain all information needed to describe exactly what changed and how it
	/// changed. It is better to provide a little bit more information than forcing a subscriber to query for
	/// additional information.
	/// 
	/// As for all Messages, it is good design practice to implement Information Messages as immutable types so 
	/// that accidental modifications are impossible. This is important since Information Messages will be transfered 
	/// to other Agents, thus, it is very likely that the content of those Information Messages will be accessed in 
	/// other threads concurrently.
	/// 
	/// In addition, Information Messages should contain copies of data (entities) or some kind of unique identifier 
	/// (identities). Latter can then be used to query for that identified entities. Never transfer object references 
	/// to private Agent data, except for global shared data that never changes; always transfer copies 
	/// of data.
	/// </summary>
	[Serializable]
	public abstract class Information : Message
	{
		#region ## Protected Interface ##

		/// <summary>
		/// Initializes a new instance.
		/// </summary>
		protected Information()
		{
		}

		#endregion

	}
}
