﻿#region Copyright (c) 2013-04, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;
using System.Diagnostics;


namespace Amarok.Agents
{
	/// <summary>
	/// </summary>
	internal sealed class CallbackSubscription : SubscriptionBase
	{
		// data
		private Action<Message> mCallback;


		#region ++ Public Interface ++

		/// <summary>
		/// </summary>
		public CallbackSubscription(Action<Message> callback, Type messageType, Predicate<Message> messageFilter,
			Action<SubscriptionBase> removeSubscriptionAction)
			: base(messageType, messageFilter, removeSubscriptionAction)
		{
			Verify.NotNull(callback, "callback");
			mCallback = callback;
		}

		#endregion

		#region ## Overridable Methods ##

		/// <summary>
		/// </summary>
		protected override void Post(Message message)
		{
			#region (asserts)
			{
				Debug.Assert(!(message is NullMessage),
					"Message of type 'NullMessage' should never be posted!!");
			}
			#endregion

			var callback = mCallback;

			if (callback != null)
			{
				callback(message);
			}
		}

		/// <summary>
		/// </summary>
		protected override void OnCompleted()
		{
			mCallback = null;

			base.OnCompleted();
		}

		#endregion

	}
}
