﻿#region Copyright (c) 2013-03, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;


namespace Amarok.Agents
{
	/// <summary>
	/// This type implements a subsystem.
	/// 
	/// Subsystems are primarily used to implement startup and loading logic by spawning agents and optionally 
	/// loading other subsystems, which again can spawn agents and load other subsystems.
	/// </summary>
	public abstract class Subsystem<TEnvironment> : Subsystem
		where TEnvironment : RuntimeEnvironment
	{
		#region ## Protected Interface ##

		/// <summary>
		/// Gets a reference to the subsystem's agent environment. The agent environment is used to give agents 
		/// access to the services of the runtime environment. One agent environment instance is shared between 
		/// multiple agents to provide the same services to all agents.
		/// </summary>
		protected new TEnvironment Environment
		{
			get
			{
				return (TEnvironment)base.Environment;
			}
		}


		/// <summary>
		/// Initializes a new instance.
		/// </summary>
		/// 
		/// <param name="environment">
		/// The runtime environment for this subsystem.</param>
		/// 
		/// <exception cref="ArgumentNullException">
		/// A null reference was passed to a method that did not accept it as a valid argument.</exception>
		protected Subsystem(TEnvironment environment)
			: base(environment)
		{
		}

		#endregion

	}
}
