﻿#region Copyright (c) 2012-10, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;
using System.Text;


namespace Amarok
{
	/// <summary>
	/// This type provides extension methods for <see cref="System.Text.Encoding"/>.
	/// </summary>
	public static class EncodingExtensions
	{
		#region ++ Public Interface (GetString) ++

		/// <summary>
		/// Decodes a sequence of bytes from the specified buffer segment into a string.
		/// </summary>
		/// 
		/// <param name="encoding">
		/// The encoding to use for decoding.</param>
		/// <param name="data">
		/// The buffer segment containing the sequence of bytes to decode.</param>
		/// 
		/// <returns>
		/// A String containing the results of decoding the specified sequence of bytes.</returns>
		public static String GetString(this Encoding encoding, BufferSegment data)
		{
			return encoding.GetString(
				data.Buffer,
				data.Offset,
				data.Count);
		}

		#endregion

	}
}
