﻿#region Copyright (c) 2012-10, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;
using System.Collections;
using System.Text;


namespace Amarok
{
	/// <summary>
	/// This type provides extension methods for <see cref="IEnumerable"/>.
	/// </summary>
	public static class EnumerableExtensions
	{
		#region ++ Public Interface (AsString) ++

		/// <summary>
		/// Concats the string representations of the items in the collection, optionally delimited by the 
		/// supplied delimiter string.
		/// </summary>
		/// 
		/// <param name="collection">
		/// The enumerable collection.</param>
		/// <param name="delimiter">
		/// An optional delimiter string. Defaults to null.</param>
		/// 
		/// <returns>
		/// A string containing the appended string representations of all items in the collection, optionally 
		/// delimited by the supplied delimiter string, or an empty string if the collection is null or empty.
		/// </returns>
		public static String AsString(this IEnumerable collection, String delimiter = null)
		{
			if (collection == null)
				return String.Empty;

			var sb = new StringBuilder();
			var count = 0;

			foreach (var item in collection)
			{
				if (count > 0)
					sb.Append(delimiter);

				if (item != null)
					sb.Append(item.ToString());

				count++;
			}

			return sb.ToString();
		}

		#endregion

	}
}
