#region Copyright (c) 2013-02, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;


namespace Amarok
{
	/// <summary>
	/// This interface represents an object with an object-id.
	/// 
	/// The object-id consists of the object's type-name and an optional instance-name. The object's type-name is 
	/// constant. In contrast, the object's instance-name can be set and changed during the object's lifetime. The 
	/// resulting object-id, a String formatted as "type-name[:instance-name]", can be used to distinguish one 
	/// object instance from other object instances.
	/// </summary>
	public interface IIdentifiableObject
	{
		/// <summary>
		/// Gets a String formatted as "type-name[:instance-name]" that represents the object-id of this object 
		/// instance. The instance-name is null by default, but can be changed throughout the object's lifetime.
		/// </summary>
		String ObjectId
		{
			get;
		}

	}
}
