﻿#region Copyright (c) 2012-10, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;
using System.ComponentModel;
using System.Runtime.InteropServices;
using System.Text;


namespace Amarok.Native
{
	/// <summary>
	/// This class contains safe platform invokes to environment-related functions.
	/// </summary>
	public static class NativeEnvironment
	{
		/// <summary>
		/// Retrieves the fully-qualified path for the file that contains the specified module. The module must 
		/// have been loaded by the current process.
		/// </summary>
		/// 
		/// <param name="moduleHandle">
		/// Handle to the loaded module whose path is being requested. If this parameter is NULL, GetModuleFileName 
		/// retrieves the path of the executable file of the current process.</param>
		/// 
		/// <returns>
		/// The fully-qualified path of the module file.</returns>
		public static String GetModuleFileName(IntPtr moduleHandle)
		{
			// call native function
			var text = new StringBuilder(2048);

			var length = (Int32)UnsafeNativeEnvironment.GetModuleFileName(
				moduleHandle,
				text,
				text.Capacity);

			var lastError = Marshal.GetLastWin32Error();

			// check result
			if (length == 0)
				throw new Win32Exception(lastError);

			return text.ToString(0, length);
		}

	}
}
