﻿#region Copyright (c) 2012-10, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;


namespace Amarok
{
	/// <summary>
	/// This type provides an implementation of <see cref="Amarok.ILazyProgress{T}"/> and 
	/// <see cref="System.IProgress{T}"/> that silently swallows all reported progress values.
	/// </summary>
	public sealed class NullProgress<T> :
		ILazyProgress<T>
	{
		#region ++ IProgress<T> Interface ++

		/// <summary>
		/// Reports a progress update.
		/// </summary>
		/// 
		/// <param name="value">
		/// The value of the updated progress.</param>
		public void Report(T value)
		{
			// ignore
		}

		#endregion

		#region ++ ILazyProgress<T> Interface ++

		/// <summary>
		/// Reports a progress update.
		/// </summary>
		/// 
		/// <param name="valueFunc">
		/// A function that returns the value of the updated progress.</param>
		/// 
		/// <exception cref="ArgumentNullException">
		/// A null reference was passed to a method that did not accept it as a valid argument.</exception>
		public void Report(Func<T> valueFunc)
		{
			Verify.NotNull(valueFunc, "valueFunc");

			// ignore
		}

		#endregion

	}
}
