﻿#region Copyright (c) 11/26/2011, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;


namespace Amarok.Presentation
{
	/// <summary>
	/// This type represents the base class for view actions related to showing message boxes.
	/// </summary>
	public abstract class ShowMessageBoxAction : ViewAction
	{
		/// <summary>
		/// Gets the message of the message box, which can contain optional placeholders.
		/// </summary>
		public String Message
		{
			get;
			private set;
		}

		/// <summary>
		/// Gets the title text of the message box.
		/// </summary>
		public String Caption
		{
			get;
			private set;
		}

		/// <summary>
		/// Gets an optional list of arguments to format into the placeholders of the message.
		/// </summary>
		public Object[] Arguments
		{
			get;
			private set;
		}


		/// <summary>
		/// Initializes a new instance.
		/// </summary>
		/// 
		/// <param name="message">
		/// The message of the message box, which can contain optional placeholders.</param>
		/// <param name="caption">
		/// The title text of the message box.</param>
		/// <param name="arguments">
		/// An optional list of arguments to format into the placeholders of the message.</param>
		/// 
		/// <exception cref="ArgumentNullException">
		/// A null reference was passed to a method that did not accept it as a valid argument.</exception>
		/// <exception cref="ArgumentException">
		/// An empty string was passed to a method that did not accept it as a valid argument.</exception>
		protected ShowMessageBoxAction(String message, String caption, params Object[] arguments)
		{
			Verify.NotEmpty(message, "message");
			Verify.NotEmpty(caption, "caption");

			this.Message = message;
			this.Caption = caption;
			this.Arguments = arguments;
		}

	}
}
