﻿#region Copyright (c) 2013-08, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;


namespace Amarok.Presentation
{
	/// <summary>
	/// This type provides hooks (events) for diagnosis purposes.
	/// </summary>
	public static class ViewCommandHooks
	{
		#region ++ Public Static Interface (Events) ++

		/// <summary>
		/// Occurs when the CanExecute() method was called.
		/// </summary>
		public static event EventHandler<ViewCommandHooksEventArgs> CanExecuteCalled;

		/// <summary>
		/// Occurs when the Execute() method was called.
		/// </summary>
		public static event EventHandler<ViewCommandHooksEventArgs> ExecuteCalled;


		/// <summary>
		/// Removes all event handler
		/// </summary>
		public static void Reset()
		{
			CanExecuteCalled = null;
			ExecuteCalled = null;
		}

		#endregion

		#region ~~ Internal Static Interface ~~

		/// <summary>
		/// </summary>
		internal static void RaiseCanExecuteCalled(Delegate handler)
		{
			try
			{
				var callback = CanExecuteCalled;

				if (callback != null)
				{
					callback(null, new ViewCommandHooksEventArgs(handler));
				}
			}
			catch (Exception)
			{
				// ignore
			}
		}

		/// <summary>
		/// </summary>
		internal static void RaiseExecuteCalled(Delegate handler)
		{
			try
			{
				var callback = ExecuteCalled;

				if (callback != null)
				{
					callback(null, new ViewCommandHooksEventArgs(handler));
				}
			}
			catch (Exception)
			{
				// ignore
			}
		}

		#endregion

		#region ++ Public Static Interface (Event Args) ++

		/// <summary>
		/// </summary>
		public sealed class ViewCommandHooksEventArgs : EventArgs
		{
			/// <summary>
			/// </summary>
			public DateTime TimestampUtc
			{
				get;
				private set;
			}

			/// <summary>
			/// </summary>
			public Delegate Handler
			{
				get;
				private set;
			}


			/// <summary>
			/// </summary>
			public ViewCommandHooksEventArgs(Delegate handler)
			{
				this.TimestampUtc = DateTime.UtcNow;
				this.Handler = handler;
			}
		}

		#endregion

	}
}
