﻿#region Copyright (c) 07/17/2012, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;
using System.Globalization;
using System.Windows;
using System.Windows.Data;


namespace Amarok.Presentation
{
	/// <summary>
	/// Converts the given String value to a Visibility value.
	/// 
	/// :: Convert
	/// Null       ...  Collapsed
	/// ""         ...  Collapsed
	/// "  "       ...  Visible
	/// Otherwise  ...  Visible
	/// 
	/// :: ConvertBack
	/// Always  ...  DependencyProperty.UnsetValue
	/// </summary>
	[ValueConversion(typeof(String), typeof(Visibility))]
	public sealed class CollapsedWhenNullOrEmptyConverter : MarkupExtensionValueConverter<
		CollapsedWhenNullOrEmptyConverter,
		String,
		Visibility>
	{
		/// <summary>
		/// </summary>
		protected override Object OnConvert(String value, Type targetType, Object parameter, CultureInfo culture)
		{
			return String.IsNullOrEmpty(value) ?
				Visibility.Collapsed :
				Visibility.Visible;
		}

		/// <summary>
		/// </summary>
		public override Object ConvertBack(Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			return DependencyProperty.UnsetValue;
		}

	}
}
