#region Copyright (c) 11/27/2011, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;
using System.Reflection;


namespace Amarok.Presentation
{
	/// <summary>
	/// </summary>
	internal abstract class HandlerInfo
	{
		#region ++ Public Interface ++

		/// <summary>
		/// </summary>
		public MethodInfo MethodInfo
		{
			get;
			private set;
		}

		/// <summary>
		/// </summary>
		public ParameterInfo ParameterInfo
		{
			get;
			private set;
		}

		/// <summary>
		/// </summary>
		public Type ViewActionType
		{
			get
			{
				return this.ParameterInfo.ParameterType;
			}
		}


		/// <summary>
		/// </summary>
		/// <param name="target"></param>
		/// <returns></returns>
		public IHandler CreateHandler(Object target)
		{
			return OnCreateHandler(target);
		}

		#endregion

		#region ## Protected Interface ##

		/// <summary>
		/// </summary>
		/// <param name="methodInfo"></param>
		/// <param name="parameterInfo"></param>
		protected HandlerInfo(MethodInfo methodInfo, ParameterInfo parameterInfo)
		{
			Verify.NotNull(methodInfo, "methodInfo");
			Verify.NotNull(parameterInfo, "parameterInfo");

			this.MethodInfo = methodInfo;
			this.ParameterInfo = parameterInfo;
		}

		#endregion

		#region ## Overridable Methods ##

		/// <summary>
		/// </summary>
		/// <param name="target"></param>
		/// <returns></returns>
		protected abstract IHandler OnCreateHandler(Object target);

		#endregion

	}
}
