﻿#region Copyright (c) 11/27/2011, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;
using System.Diagnostics;
using System.Globalization;


namespace Amarok.Presentation
{
	/// <summary>
	/// </summary>
	/// <typeparam name="TViewAction"></typeparam>
	internal sealed class ViewActionHandler<TViewAction> :
		IHandler
		where TViewAction : ViewAction
	{
		#region ++ IHandler Interface ++

		/// <summary>
		/// </summary>
		/// <param name="viewAction"></param>
		Boolean IHandler.Invoke(ViewAction viewAction)
		{
			var action = viewAction as TViewAction;
			Debug.Assert(action != null, "Programming Error: viewAction must be assignable to TViewAction!");

			try
			{
				// call handler method
				return this.Func(action);
			}
			catch (Exception exception)
			{
				#region (tracing)
				{
					Trace.WriteLine(String.Format(CultureInfo.InvariantCulture,
						"*** WARNING: UNHANDLED VIEW ACTION HANDLER EXCEPTION:\r\n" +
						"  ViewActionType={0}\r\n" +
						"  ViewAction={1}\r\n{2}",
						viewAction.GetType(),
						viewAction,
						Helper.FormatException(exception, CultureInfo.InvariantCulture)));
				}
				#endregion

				// not handled
				return false;
			}
		}

		#endregion

		#region ++ Public Interface ++

		/// <summary>
		/// </summary>
		public Func<TViewAction, Boolean> Func
		{
			get;
			private set;
		}


		/// <summary>
		/// </summary>
		/// <param name="func"></param>
		public ViewActionHandler(Func<TViewAction, Boolean> func)
		{
			Verify.NotNull(func, "func");
			this.Func = func;
		}

		#endregion

	}
}
