﻿#region Copyright (c) 2013-02, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;


namespace Amarok.Presentation
{
	/// <summary>
	/// This type provides a base class for view model implementations. This base class provides rich support for 
	/// property change notifications. If you need support for validation data errors too, you should derive from 
	/// base class <see cref="ValidatingViewModel"/> instead.
	/// </summary>
	public abstract class ViewModel<TParent> : ViewModel
		where TParent : ViewModel
	{
		#region ++ Public Interface (Parent) ++

		/// <summary>
		/// A reference to the parent object, or null if no parent object exists.
		/// </summary>
		public new TParent Parent
		{
			get
			{
				return (TParent)base.Parent;
			}
			set
			{
				base.Parent = value;
			}
		}

		#endregion

		#region ## Protected Interface ##

		/// <summary>
		/// Initializes a new instance.
		/// </summary>
		/// 
		/// <param name="parent">
		/// A reference to the parent of this view model object.</param>
		/// 
		/// <exception cref="ArgumentNullException">
		/// A null reference was passed to a method that did not accept it as a valid argument.</exception>
		protected ViewModel(TParent parent)
			: base(parent)
		{
		}

		/// <summary>
		/// Initializes a new instance.
		/// </summary>
		protected ViewModel()
		{
		}

		#endregion

	}
}
