#region Copyright (c) 11/26/2011, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

using System;
using System.Windows;


namespace Amarok.Presentation
{
	/// <summary>
	/// This type contains state information and event data associated with a ViewAction routed event.
	/// </summary>
	public class ViewActionRoutedEventArgs : RoutedEventArgs
	{
		/// <summary>
		/// Gets the view action associated with this routed event.
		/// </summary>
		public ViewAction ViewAction
		{
			get;
			private set;
		}


		/// <summary>
		/// Initializes a new instance.
		/// </summary>
		/// 
		/// <param name="routedEvent">
		/// The routed event identifier for this instance of the RoutedEventArgs class.</param>
		/// <param name="source">
		/// An alternate source that will be reported when the event is handled.</param>
		/// <param name="viewAction">
		/// The view action associated with this routed event.</param>
		/// 
		/// <exception cref="ArgumentNullException">
		/// A null reference was passed to a method that did not accept it as a valid argument.</exception>
		public ViewActionRoutedEventArgs(RoutedEvent routedEvent, Object source, ViewAction viewAction)
			: base(routedEvent, source)
		{
			Verify.NotNull(viewAction, "viewAction");
			this.ViewAction = viewAction;
		}

	}
}
