﻿#region Copyright (c) 2013-04, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using NUnit.Framework;


namespace Amarok.Agents
{
	[TestFixture]
	public class Test_Agent_TEnvironment_TOptions
	{
		private class MyAgentEnvironment : AgentEnvironment
		{
		}

		private class MyAgentOptions : AgentOptions
		{
		}

		private class MyAgent : Agent<MyAgentEnvironment, MyAgentOptions>
		{
			public MyAgent(MyAgentEnvironment environment, MyAgentOptions options)
				: base(environment, options)
			{
			}

			public MyAgentEnvironment TestEnvironment
			{
				get
				{
					return base.Environment;
				}
			}

			public MyAgentOptions TestOptions
			{
				get
				{
					return base.Options;
				}
			}
		}


		// --- TESTS ---


		[Test]
		public void Test_Construction_NullEnvironment()
		{
			Assert.Throws<ArgumentNullException>(() =>
				{
					var agent = new MyAgent(null, new MyAgentOptions());
				});
		}

		[Test]
		public void Test_Construction_NullOptions()
		{
			Assert.Throws<ArgumentNullException>(() =>
				{
					var agent = new MyAgent(new MyAgentEnvironment(), null);
				});
		}

		[Test]
		public void Test_Construction()
		{
			var environment = new MyAgentEnvironment();
			var options = new MyAgentOptions();
			var agent = new MyAgent(environment, options);

			Assert.That(agent.TestEnvironment, Is.SameAs(environment));
			Assert.That(agent.TestOptions, Is.SameAs(options));
		}

	}
}

#endif
