﻿#region Copyright (c) 2013-04, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using NUnit.Framework;


namespace Amarok.Agents
{
	public partial class Test_Agent
	{
		[Test]
		public void Test_AgentProxy_Publish_SingleMessage()
		{
			var environment = new MyAgentEnvironment();
			var options = new MyAgentOptions();
			var agent = new MyAgent(environment, options);
			var agentInterface = (IAgent)agent;
			var agentProxy = (IAgentProxy)agent;
			var subscriber = new MySubscriberAgent(environment, options);

			agentInterface.Subscribe(
				typeof(TestMessage),
				subscriber.ToAgentReference());

			Assert.That(subscriber.MessageHandlerCalled, Is.EqualTo(0));

			var message = new TestMessage();
			agentProxy.Publish(message);

			Helper.WaitForCondition(5000, 10, () => subscriber.MessageHandlerCalled == 1);
			Assert.That(subscriber.MessageHandlerCalled, Is.EqualTo(1));
			Assert.That(message.Label, Is.Null);
		}

		[Test]
		public void Test_AgentProxy_Post_SingleMessage()
		{
			var environment = new MyAgentEnvironment();
			var options = new MyAgentOptions();
			var agent = new MyAgent(environment, options);
			var agentProxy = (IAgentProxy)agent;

			Assert.That(agent.MessageHandlerCalled, Is.EqualTo(0));

			var message = new TestMessage();
			agentProxy.Post(message);

			Helper.WaitForCondition(5000, 10, () => agent.MessageHandlerCalled == 1);
			Assert.That(agent.MessageHandlerCalled, Is.EqualTo(1));
		}

	}
}

#endif
