﻿#region Copyright (c) 2013-04, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using System.Threading.Tasks;
using NUnit.Framework;


namespace Amarok.Agents
{
	[TestFixture]
	public class Test_AgentExtensions
	{
		private class MyAgent : IAgent
		{
			public Boolean Post(Message message)
			{
				throw new NotImplementedException();
			}

			public void Dispose()
			{
				throw new NotImplementedException();
			}

			public Task Completion
			{
				get
				{
					throw new NotImplementedException();
				}
			}

			public void Complete(Boolean discardPendingMessages = false)
			{
				throw new NotImplementedException();
			}

			public ICompletable Subscribe(Type messageType, AgentReference subscriber, Predicate<Message> messageFilter = null)
			{
				throw new NotImplementedException();
			}

			public ICompletable Subscribe(Type messageType, Action<Message> subscriber, Predicate<Message> messageFilter = null)
			{
				throw new NotImplementedException();
			}
		}


		// --- TESTS ---


		#region ToAgentReference

		[Test]
		public void Test_ToAgentReference_Null()
		{
			var agent = (IAgent)null;
			var agentReference = agent.ToAgentReference();

			Assert.That(agentReference, Is.Null);
		}

		[Test]
		public void Test_ToAgentReference_Agent()
		{
			var agent = new MyAgent();
			var agentReference = agent.ToAgentReference();

			Assert.That(agentReference, Is.Not.Null);
			Assert.That(agentReference.IsAlive, Is.True);
			Assert.That(agentReference.Target, Is.SameAs(agent));

			GC.KeepAlive(agent);
		}

		[Test]
		public void Test_ToAgentReference_AgentReference()
		{
			var agent = new MyAgent();
			var agentRef = new AgentReference(agent);
			var agentReference = agentRef.ToAgentReference();

			Assert.That(agentReference, Is.Not.Null);
			Assert.That(agentReference.IsAlive, Is.True);
			Assert.That(agentReference.Target, Is.SameAs(agent));
			Assert.That(agentReference, Is.SameAs(agentRef));

			GC.KeepAlive(agent);
		}

		#endregion

	}
}

#endif
