﻿#region Copyright (c) 2013-04, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using NUnit.Framework;


namespace Amarok.Agents
{
	[TestFixture]
	public class Test_AgentFactoryExtensions
	{
		private class StubAgent : Agent
		{
			public StubAgent(AgentEnvironment environment, AgentOptions options)
				: base(environment, options)
			{
			}
		}

		private class StubGenericAgent : Agent<StubEnvironment, StubOptions>
		{
			public StubGenericAgent(StubEnvironment environment, StubOptions options)
				: base(environment, options)
			{
			}
		}

		private class StubEnvironment : AgentEnvironment
		{
		}

		private class StubOptions : AgentOptions
		{
		}

		private class MockFactory : IAgentFactory
		{
			public Int32 Called;
			public Type TypeArgument;
			public AgentEnvironment EnvironmentArgument;
			public AgentOptions OptionsArgument;
			public StubAgent ReturnValue;

			public IAgent SpawnAgent(Type agentType, AgentEnvironment environment, AgentOptions options)
			{
				TypeArgument = agentType;
				EnvironmentArgument = environment;
				OptionsArgument = options;
				Called++;

				ReturnValue = new StubAgent(environment, options);
				return ReturnValue;
			}
		}


		// --- TESTS ---


		#region SpawnAgent(Type, AgentEnvironment)

		[Test]
		public void Test_SpawnAgent_Type_Environment()
		{
			var environment = new StubEnvironment();
			var factory = new MockFactory();

			var agent = factory.SpawnAgent(
				typeof(StubAgent),
				environment);

			Assert.That(factory.Called, Is.EqualTo(1));
			Assert.That(factory.TypeArgument, Is.EqualTo(typeof(StubAgent)));
			Assert.That(factory.EnvironmentArgument, Is.SameAs(environment));
			Assert.That(factory.OptionsArgument, Is.SameAs(AgentOptions.Empty));
			Assert.That(agent, Is.SameAs(factory.ReturnValue));
		}

		#endregion

		#region SpawnAgent<T>(AgentEnvironment, AgentOptions)

		[Test]
		public void Test_SpawnAgent_GenericType_Environment_Options()
		{
			var environment = new StubEnvironment();
			var options = new StubOptions();
			var factory = new MockFactory();

			var agent = factory.SpawnAgent<StubAgent>(
				environment,
				options);

			Assert.That(factory.Called, Is.EqualTo(1));
			Assert.That(factory.TypeArgument, Is.EqualTo(typeof(StubAgent)));
			Assert.That(factory.EnvironmentArgument, Is.SameAs(environment));
			Assert.That(factory.OptionsArgument, Is.SameAs(options));
			Assert.That(agent, Is.SameAs(factory.ReturnValue));
		}

		#endregion

		#region SpawnAgent<T>(AgentEnvironment)

		[Test]
		public void Test_SpawnAgent_GenericType_Environment()
		{
			var environment = new StubEnvironment();
			var factory = new MockFactory();

			var agent = factory.SpawnAgent<StubAgent>(
				environment);

			Assert.That(factory.Called, Is.EqualTo(1));
			Assert.That(factory.TypeArgument, Is.EqualTo(typeof(StubAgent)));
			Assert.That(factory.EnvironmentArgument, Is.SameAs(environment));
			Assert.That(factory.OptionsArgument, Is.SameAs(AgentOptions.Empty));
			Assert.That(agent, Is.SameAs(factory.ReturnValue));
		}

		#endregion

		#region SpawnAgent<T, E, O>(AgentEnvironment, AgentOptions)

		[Test]
		public void Test_SpawnAgent_GenericType_GenericEnvironment_GenericOptions()
		{
			var environment = new StubEnvironment();
			var options = new StubOptions();
			var factory = new MockFactory();

			var agent = factory.SpawnAgent<StubGenericAgent, StubEnvironment, StubOptions>(
				environment,
				options);

			Assert.That(factory.Called, Is.EqualTo(1));
			Assert.That(factory.TypeArgument, Is.EqualTo(typeof(StubGenericAgent)));
			Assert.That(factory.EnvironmentArgument, Is.SameAs(environment));
			Assert.That(factory.OptionsArgument, Is.SameAs(options));
			Assert.That(agent, Is.SameAs(factory.ReturnValue));
		}

		#endregion

	}
}

#endif
