﻿#region Copyright (c) 2013-04, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using NUnit.Framework;


namespace Amarok.Agents
{
	[TestFixture]
	public sealed class Test_Agent_Case27
	{
		private class DisposeChildAgent : Operation<None>
		{
		}

		private class ChildAgent : Agent
		{
			public ChildAgent(AgentEnvironment environment, AgentOptions options)
				: base(environment, options)
			{
			}
		}

		private class SupervisorAgent : Agent
		{
			public static Nullable<Boolean> WaitResult;

			private IAgent mChildAgent;


			public SupervisorAgent(AgentEnvironment environment, AgentOptions options)
				: base(environment, options)
			{
				mChildAgent = Agent.Factory.SpawnAgent<ChildAgent>(environment);
			}

			[OperationHandler, ExclusiveExecution, OrderedProcessing]
			private None _HandleDisposeChildAgent(DisposeChildAgent operation)
			{
				mChildAgent.Complete(true);
				WaitResult = mChildAgent.Completion.Wait(5000);

				return None.Instance;
			}
		}


		// --- TESTS ---


		[Test]
		public void Test_Case27()
		{
			SupervisorAgent.WaitResult = null;

			var environment = new AgentEnvironment();
			var supervisorAgent = Agent.Factory.SpawnAgent<SupervisorAgent>(environment);

			supervisorAgent.PostOperation(new DisposeChildAgent())
				.WaitQuietly();

			Assert.That(SupervisorAgent.WaitResult, Is.True);
		}

	}
}

#endif
