﻿#region Copyright (c) 2013-04, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using System.Threading;
using NUnit.Framework;


namespace Amarok.Agents
{
	[TestFixture]
	public sealed class Test_Agent_Case66
	{
		private class TestMessage : Operation<None>
		{
		}

		private class TestAgent : Agent
		{
			public Int32 HandlerCalled;

			public TestAgent(AgentEnvironment environment, AgentOptions options)
				: base(environment, options)
			{
			}

			[MessageHandler, ExclusiveExecution, OrderedProcessing]
			private void _HandleMessage(Message message)
			{
				HandlerCalled++;
			}
		}


		// --- TESTS ---


		[Test]
		public void Test_Case66_Agent_Connected_Before_Messages_Published()
		{
			var environment = new AgentEnvironment();
			var options = new AgentOptions();
			var messageBus = new DefaultMessageBus();
			var agent = new TestAgent(environment, options);

			Assert.That(agent.HandlerCalled, Is.EqualTo(0));

			messageBus.Connect(agent);

			Assert.That(agent.HandlerCalled, Is.EqualTo(0), "major flaw");

			messageBus.PublishOperation(new TestMessage()).Wait(5000);

			Assert.That(agent.HandlerCalled, Is.EqualTo(1));
		}

		[Test]
		public void Test_Case66_Agent_Connected_After_Messages_Published()
		{
			var environment = new AgentEnvironment();
			var options = new AgentOptions();
			var messageBus = new DefaultMessageBus();
			var agent = new TestAgent(environment, options);

			Assert.That(agent.HandlerCalled, Is.EqualTo(0));

			messageBus.PublishOperation(new TestMessage()).Wait(100);

			Assert.That(agent.HandlerCalled, Is.EqualTo(0));

			messageBus.Connect(agent);
			Thread.Sleep(250);

			Assert.That(agent.HandlerCalled, Is.EqualTo(0), "major flaw");

			messageBus.PublishOperation(new TestMessage()).Wait(5000);

			Assert.That(agent.HandlerCalled, Is.EqualTo(1));
		}

	}
}

#endif
