﻿#region Copyright (c) 2012-11, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using NUnit.Framework;


namespace Amarok.Agents
{
	[TestFixture]
	public class Test_Information
	{
		[Serializable]
		public class MyInformation : Information
		{
			public Int32 Value
			{
				get;
				private set;
			}

			public MyInformation(Int32 value)
			{
				this.Value = value;
			}
		}


		// --- TESTS ---


		[Test]
		public void Test_Construction()
		{
			var instance = new MyInformation(123);

			Assert.That(instance.Value, Is.EqualTo(123));
			Assert.That(instance.Id, Is.Not.EqualTo(Guid.Empty));
			Assert.That(instance.Label, Is.Null);
			Assert.That(instance.HasLabel, Is.False);
			Assert.That(instance.ToString(), Is.StringContaining("Amarok.Agents.Test_Information+MyInformation("));
			Assert.That(instance.ToString(), Is.StringContaining("Id: "));
			Assert.That(instance.ToString(), Is.StringContaining(", Label: <null>)"));
		}

		[Test]
		public void Test_Serialization_BinaryFormatter_WithoutLabel()
		{
			var instance1 = new MyInformation(123);
			var instance2 = TestHelper.TestBinaryFormatterRoundtrip(instance1);

			Assert.That(instance2, Is.Not.SameAs(instance1));
			Assert.That(instance2.Value, Is.EqualTo(123));
			Assert.That(instance2.Label, Is.Null);
			Assert.That(instance2.HasLabel, Is.False);
			Assert.That(instance2.Id, Is.EqualTo(instance1.Id));
		}

		[Test]
		public void Test_Serialization_BinaryFormatter_WithLabel()
		{
			var instance1 = new MyInformation(123);
			instance1.Label = "COM1";
			var instance2 = TestHelper.TestBinaryFormatterRoundtrip(instance1);

			Assert.That(instance2, Is.Not.SameAs(instance1));
			Assert.That(instance2.Value, Is.EqualTo(123));
			Assert.That(instance2.Label, Is.EqualTo("COM1"));
			Assert.That(instance2.HasLabel, Is.True);
			Assert.That(instance2.Id, Is.EqualTo(instance1.Id));
		}

		[Test]
		public void Test_Serialization_DataContract_WithoutLabel()
		{
			var instance1 = new MyInformation(123);
			var instance2 = TestHelper.TestDataContractRoundtrip(instance1);

			Assert.That(instance2, Is.Not.SameAs(instance1));
			Assert.That(instance2.Value, Is.EqualTo(123));
			Assert.That(instance2.Label, Is.Null);
			Assert.That(instance2.HasLabel, Is.False);
			Assert.That(instance2.Id, Is.EqualTo(instance1.Id));
		}

		[Test]
		public void Test_Serialization_DataContract_WithLabel()
		{
			var instance1 = new MyInformation(123);
			instance1.Label = "COM1";
			var instance2 = TestHelper.TestDataContractRoundtrip(instance1);

			Assert.That(instance2, Is.Not.SameAs(instance1));
			Assert.That(instance2.Value, Is.EqualTo(123));
			Assert.That(instance2.Label, Is.EqualTo("COM1"));
			Assert.That(instance2.HasLabel, Is.True);
			Assert.That(instance2.Id, Is.EqualTo(instance1.Id));
		}

	}
}

#endif
