﻿#region Copyright (c) 2013-03, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using NUnit.Framework;


namespace Amarok.Agents
{
	[TestFixture]
	public class Test_RuntimeStartException
	{
		[Test]
		public void Test_Construction()
		{
			var exception = new RuntimeStartException();

			Assert.That(exception.Message, Is.Not.Null);
			Assert.That(exception.InnerException, Is.Null);
		}

		[Test]
		public void Test_Construction_Message()
		{
			var exception = new RuntimeStartException("Message");

			Assert.That(exception.Message, Is.EqualTo("Message"));
			Assert.That(exception.InnerException, Is.Null);
		}

		[Test]
		public void Test_Construction_Message_Exception()
		{
			var innerException = new ApplicationException("Test");
			var exception = new RuntimeStartException("Message", innerException);

			Assert.That(exception.Message, Is.EqualTo("Message"));
			Assert.That(exception.InnerException, Is.SameAs(innerException));
		}

		[Test]
		public void Test_Serialization()
		{
			var innerException = new ApplicationException("Test");
			var originalException = new RuntimeStartException("Message", innerException);

			var exception = TestHelper.TestBinaryFormatterRoundtrip<RuntimeStartException>(originalException);

			Assert.That(exception.Message, Is.EqualTo("Message"));
			Assert.That(exception.InnerException, Is.TypeOf<ApplicationException>());
			Assert.That(exception.InnerException.Message, Is.EqualTo("Test"));
		}

	}
}

#endif
