﻿#region Copyright (c) 2012-10, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using NUnit.Framework;


namespace Amarok
{
	[TestFixture]
	public class Test_Entity
	{
		[Serializable]
		private class TestIdentity : Identity
		{
		}

		[Serializable]
		private class TestEntity : Entity
		{
			public TestEntity(TestIdentity identity)
				: base(identity)
			{
			}
		}


		// --- TESTS ---


		[Test]
		public void Test_Construction()
		{
			var identity = new TestIdentity();
			var entity = new TestEntity(identity);

			Assert.That(entity.Identity, Is.SameAs(identity));
		}

		[Test]
		public void Test_Construction_NullArgument()
		{
			Assert.Throws<ArgumentNullException>(() =>
				{
					var entity = new TestEntity(null);
				});
		}

		[Test]
		public void Test_ImplicitCast_Identity()
		{
			var identity = new TestIdentity();
			var entity = new TestEntity(identity);

			Identity castedIdentity = entity;

			Assert.That(castedIdentity, Is.SameAs(identity));
		}

		[Test]
		public void Test_ImplicitCast_Identity_Null()
		{
			var entity = (TestEntity)null;

			Identity castedIdentity = entity;

			Assert.That(castedIdentity, Is.Null);
		}

		[Test]
		public void Test_ToIdentity()
		{
			var identity = new TestIdentity();
			var entity = new TestEntity(identity);

			Assert.That(entity.ToIdentity(), Is.SameAs(identity));
		}

		[Test]
		public void Test_Serialization_BinaryFormatter()
		{
			var objectA = new TestEntity(new TestIdentity());
			var objectB = TestHelper.TestBinaryFormatterRoundtrip<TestEntity>(objectA);

			Assert.That(objectB, Is.Not.SameAs(objectA));
			Assert.That(objectB.Identity, Is.Not.SameAs(objectA.Identity));
		}

		[Test]
		public void Test_Serialization_DataContract()
		{
			var objectA = new TestEntity(new TestIdentity());

			var objectB = TestHelper.TestDataContractRoundtrip<TestEntity>(
				objectA,
				typeof(TestIdentity)
			);

			Assert.That(objectB, Is.Not.SameAs(objectA));
			Assert.That(objectB.Identity, Is.Not.SameAs(objectA.Identity));
		}

	}
}

#endif
