﻿#region Copyright (c) 2013-02, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using NUnit.Framework;


namespace Amarok
{
	[TestFixture]
	public class Test_IdentifiableObject
	{
		private class TestClass : IdentifiableObject
		{
			public static readonly String TypeName = typeof(TestClass).FullName;


			public TestClass()
			{
			}

			public TestClass(String instanceName)
				: base(instanceName)
			{
			}

			public String TestObjectId
			{
				get
				{
					return base.ObjectId;
				}
			}

			public void TestSetInstanceName(String instanceName)
			{
				this.SetInstanceName(instanceName);
			}

			protected override void SetInstanceName(String instanceName)
			{
				base.SetInstanceName(instanceName);
			}
		}


		// --- TESTS ---


		[Test]
		public void Test_Construction()
		{
			var test = new TestClass();
			var testInterface = (IIdentifiableObject)test;

			Assert.That(test.TestObjectId, Is.EqualTo(TestClass.TypeName));
			Assert.That(test.ToString(), Is.EqualTo(TestClass.TypeName));
			Assert.That(testInterface.ObjectId, Is.EqualTo(TestClass.TypeName));

			test.TestSetInstanceName("BLAH");

			Assert.That(test.TestObjectId, Is.EqualTo(TestClass.TypeName + ":BLAH"));
			Assert.That(test.ToString(), Is.EqualTo(TestClass.TypeName + ":BLAH"));
			Assert.That(testInterface.ObjectId, Is.EqualTo(TestClass.TypeName + ":BLAH"));

			test.TestSetInstanceName("");

			Assert.That(test.TestObjectId, Is.EqualTo(TestClass.TypeName));
			Assert.That(test.ToString(), Is.EqualTo(TestClass.TypeName));
			Assert.That(testInterface.ObjectId, Is.EqualTo(TestClass.TypeName));

			test.TestSetInstanceName(null);

			Assert.That(test.TestObjectId, Is.EqualTo(TestClass.TypeName));
			Assert.That(test.ToString(), Is.EqualTo(TestClass.TypeName));
			Assert.That(testInterface.ObjectId, Is.EqualTo(TestClass.TypeName));
		}

		[Test]
		public void Test_Construction_InstanceName()
		{
			var test = new TestClass("COM1");
			var testInterface = (IIdentifiableObject)test;

			Assert.That(test.TestObjectId, Is.EqualTo(TestClass.TypeName + ":COM1"));
			Assert.That(test.ToString(), Is.EqualTo(TestClass.TypeName + ":COM1"));
			Assert.That(testInterface.ObjectId, Is.EqualTo(TestClass.TypeName + ":COM1"));

			test.TestSetInstanceName("BLAH");

			Assert.That(test.TestObjectId, Is.EqualTo(TestClass.TypeName + ":BLAH"));
			Assert.That(test.ToString(), Is.EqualTo(TestClass.TypeName + ":BLAH"));
			Assert.That(testInterface.ObjectId, Is.EqualTo(TestClass.TypeName + ":BLAH"));

			test.TestSetInstanceName("");

			Assert.That(test.TestObjectId, Is.EqualTo(TestClass.TypeName));
			Assert.That(test.ToString(), Is.EqualTo(TestClass.TypeName));
			Assert.That(testInterface.ObjectId, Is.EqualTo(TestClass.TypeName));

			test.TestSetInstanceName(null);

			Assert.That(test.TestObjectId, Is.EqualTo(TestClass.TypeName));
			Assert.That(test.ToString(), Is.EqualTo(TestClass.TypeName));
			Assert.That(testInterface.ObjectId, Is.EqualTo(TestClass.TypeName));
		}

	}
}

#endif
