﻿#region Copyright (c) 2013-02, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using NUnit.Framework;


namespace Amarok
{
	[TestFixture]
	public class Test_IdentifiableObject_Performance
	{
		private class TestIdentifiableObject : IdentifiableObject
		{
			public String TestObjectId
			{
				get
				{
					return base.ObjectId;
				}
			}
		}

		private class TestNonIdentifiableObject
		{
		}


		// --- TESTS ---


		[Test]
		public void Test_Performance()
		{
			TestHelper.TestPerformance("NonIdentifiableObject", 1000000, i =>
				{
					var obj = new TestNonIdentifiableObject();
					Assert.That(obj, Is.Not.Null);
				});
			GC.Collect(2, GCCollectionMode.Forced);
			GC.WaitForPendingFinalizers();

			TestHelper.TestPerformance("IdentifiableObject", 1000000, i =>
				{
					var obj = new TestIdentifiableObject();
					Assert.That(obj, Is.Not.Null);
				});
			GC.Collect(2, GCCollectionMode.Forced);
			GC.WaitForPendingFinalizers();

			var test = new TestIdentifiableObject();
			TestHelper.TestPerformance("Get-ObjectId", 1000000, i =>
				{
					Assert.That(test.TestObjectId, Is.Not.Null);
				});
		}

	}
}

#endif
