﻿#region Copyright (c) 2012-10, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using System.Collections.Generic;
using NUnit.Framework;


namespace Amarok
{
	[TestFixture]
	public class Test_StringDictionary
	{
		[Test]
		public void Test_Construction()
		{
			var dict = new StringDictionary<Int32>(StringComparer.OrdinalIgnoreCase);

			dict["A"] = 111;
			dict["B"] = 222;

			Assert.That(dict["A"], Is.EqualTo(111));
			Assert.That(dict["a"], Is.EqualTo(111));
			Assert.That(dict["B"], Is.EqualTo(222));
			Assert.That(dict["b"], Is.EqualTo(222));
		}

		[Test]
		public void Test_Construction_NullComparer()
		{
			Assert.Throws<ArgumentNullException>(() =>
				{
					var dict = new StringDictionary<Int32>(null);
				});
		}

		[Test]
		public void Test_Construction_Capacity()
		{
			var dict = new StringDictionary<Int32>(32, StringComparer.OrdinalIgnoreCase);

			dict["A"] = 111;
			dict["B"] = 222;

			Assert.That(dict["A"], Is.EqualTo(111));
			Assert.That(dict["a"], Is.EqualTo(111));
			Assert.That(dict["B"], Is.EqualTo(222));
			Assert.That(dict["b"], Is.EqualTo(222));
		}

		[Test]
		public void Test_Construction_Capacity_NullComparer()
		{
			Assert.Throws<ArgumentNullException>(() =>
				{
					var dict = new StringDictionary<Int32>(64, null);
				});
		}

		[Test]
		public void Test_Construction_Collection()
		{
			var dict1 = new Dictionary<String, Int32>();
			dict1.Add("A", 111);
			dict1.Add("B", 222);

			var dict2 = new StringDictionary<Int32>(dict1, StringComparer.OrdinalIgnoreCase);

			Assert.That(dict2["A"], Is.EqualTo(111));
			Assert.That(dict2["a"], Is.EqualTo(111));
			Assert.That(dict2["B"], Is.EqualTo(222));
			Assert.That(dict2["b"], Is.EqualTo(222));
		}

		[Test]
		public void Test_Construction_Collection_NullCollection()
		{
			Assert.Throws<ArgumentNullException>(() =>
				{
					var dict2 = new StringDictionary<Int32>(null, StringComparer.OrdinalIgnoreCase);
				});
		}

		[Test]
		public void Test_Construction_Collection_NullComparer()
		{
			var dict1 = new Dictionary<String, Int32>();
			dict1.Add("A", 111);
			dict1.Add("B", 222);

			Assert.Throws<ArgumentNullException>(() =>
				{
					var dict2 = new StringDictionary<Int32>(dict1, null);
				});
		}

		[Test]
		public void Test_Serialization_BinaryFormatter()
		{
			var dict1 = new StringDictionary<Int32>(StringComparer.OrdinalIgnoreCase);

			dict1["A"] = 111;
			dict1["B"] = 222;

			var dict2 = TestHelper.TestBinaryFormatterRoundtrip<StringDictionary<Int32>>(dict1);

			Assert.That(dict2, Is.Not.SameAs(dict1));
			Assert.That(dict2["A"], Is.EqualTo(111));
			Assert.That(dict2["a"], Is.EqualTo(111));
			Assert.That(dict2["B"], Is.EqualTo(222));
			Assert.That(dict2["b"], Is.EqualTo(222));
		}

	}
}

#endif
