﻿#region Copyright (c) 2012-10, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using NUnit.Framework;


namespace Amarok
{
	[TestFixture]
	public class Test_TypeDictionary
	{
		[Test]
		public void Test_Constrcutor()
		{
			var dict = new TypeDictionary<String>();

			dict.Add(typeof(Int32), "int");
			dict.Add(typeof(Int64), "long");

			Assert.That(dict[typeof(Int32)], Is.EqualTo("int"));
			Assert.That(dict[typeof(Int64)], Is.EqualTo("long"));
		}

		[Test]
		public void Test_Constrcutor_Capacity()
		{
			var dict = new TypeDictionary<String>(24);

			dict.Add(typeof(Int32), "int");
			dict.Add(typeof(Int64), "long");

			Assert.That(dict[typeof(Int32)], Is.EqualTo("int"));
			Assert.That(dict[typeof(Int64)], Is.EqualTo("long"));
		}

		[Test]
		public void Test_Constrcutor_Collection()
		{
			var dict1 = new TypeDictionary<String>();

			dict1.Add(typeof(Int32), "int");
			dict1.Add(typeof(Int64), "long");

			var dict2 = new TypeDictionary<String>(dict1);

			Assert.That(dict2[typeof(Int32)], Is.EqualTo("int"));
			Assert.That(dict2[typeof(Int64)], Is.EqualTo("long"));
		}

		[Test]
		public void Test_Serialization_BinaryFormatter()
		{
			var dict1 = new TypeDictionary<String>();

			dict1.Add(typeof(Int32), "int");
			dict1.Add(typeof(Int64), "long");

			var dict2 = TestHelper.TestBinaryFormatterRoundtrip<TypeDictionary<String>>(dict1);

			Assert.That(dict2, Is.Not.SameAs(dict1));
			Assert.That(dict2[typeof(Int32)], Is.EqualTo("int"));
			Assert.That(dict2[typeof(Int64)], Is.EqualTo("long"));
		}

	}
}

#endif
