﻿#region Copyright (c) 2012-10, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using System.Collections;
using NUnit.Framework;


namespace Amarok
{
	[TestFixture]
	public class Test_EnumerableExtensions
	{
		#region AsString

		[Test]
		public void Test_AsString_Null()
		{
			var collection = (IEnumerable)null;
			Assert.That(collection.AsString(), Is.EqualTo(String.Empty));
		}

		[Test]
		public void Test_AsString_Empty()
		{
			var collection = new ArrayList();
			Assert.That(collection.AsString(), Is.EqualTo(String.Empty));
		}

		[Test]
		public void Test_AsString_SingleItem_WithoutDelimiter()
		{
			var collection = new Int32[] { 123 };
			Assert.That(collection.AsString(), Is.EqualTo("123"));
		}

		[Test]
		public void Test_AsString_SingleItem_WithDelimiter()
		{
			var collection = new Int32[] { 123 };
			Assert.That(collection.AsString(";"), Is.EqualTo("123"));
		}

		[Test]
		public void Test_AsString_MultipleItems_WithoutDelimiter()
		{
			var collection = new Int32[] { 123, 456, 789 };
			Assert.That(collection.AsString(), Is.EqualTo("123456789"));
		}

		[Test]
		public void Test_AsString_MultipleItems_WithDelimiter()
		{
			var collection = new Int32[] { 123, 456, 789 };
			Assert.That(collection.AsString(";"), Is.EqualTo("123;456;789"));
		}

		[Test]
		public void Test_AsString_MultipleItems_WithDelimiter_EmptyString()
		{
			var collection = new Int32[] { 123, 456, 789 };
			Assert.That(collection.AsString(""), Is.EqualTo("123456789"));
		}

		[Test]
		public void Test_AsString_MultipleItems_NullItems()
		{
			var collection = new String[] { null, "123", "456", null, "789", null };
			Assert.That(collection.AsString(";"), Is.EqualTo(";123;456;;789;"));
		}

		#endregion

	}
}

#endif
