﻿#region Copyright (c) 2012-10, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using System.IO;
using NUnit.Framework;


namespace Amarok
{
	[TestFixture]
	public class Test_StreamExtensions
	{
		#region Write

		[Test]
		public void Test_Write()
		{
			var buffer = new Byte[] { 0x20, 0x30, 0x31, 0x32, 0x20, 0x20 };
			var segment = new BufferSegment(buffer, 1, 3);
			var stream = new MemoryStream();

			stream.Write(segment);

			Assert.That(stream.ToArray(), Is.EquivalentTo(new Byte[] { 0x30, 0x31, 0x32 }));
		}

		#endregion

		#region ReadToEnd

		[Test]
		public void Test_ReadToEnd()
		{
			var buffer = new Byte[] { 0x20, 0x30, 0x31, 0x32, 0x20, 0x20 };
			var stream = new MemoryStream(buffer, 1, 3);
			var segment = stream.ReadToEnd();

			Assert.That(segment.ToArray(), Is.EquivalentTo(new Byte[] { 0x30, 0x31, 0x32 }));
		}

		[Test]
		public void Test_ReadToEnd_EmptyStream()
		{
			var stream = new MemoryStream();
			var segment = stream.ReadToEnd();

			Assert.That(segment, Is.EqualTo(BufferSegment.Empty));
		}

		#endregion

	}
}

#endif
