﻿#region Copyright (c) 2012-10, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using System.ComponentModel;
using NUnit.Framework;


namespace Amarok.Native
{
	[TestFixture]
	public class Test_NativeEnvironment
	{
		[Test]
		public void TestGetModuleFileName()
		{
			var moduleFileName = NativeEnvironment.GetModuleFileName(IntPtr.Zero);

			Assert.That(moduleFileName, Is.Not.Null);
			Assert.That(moduleFileName, Is.Not.Empty);
		}

		[Test]
		public void TestGetModuleFileName_InvalidHandle()
		{
			var exception = Assert.Throws<Win32Exception>(() =>
				{
					NativeEnvironment.GetModuleFileName(new IntPtr(-999));
				});

			if (TestHelper.IsWindowsVista)
			{
				// Windows Vista does not return a last error
				Assert.That(exception.NativeErrorCode, Is.EqualTo(0));
				Assert.That(exception.Message, Is.Not.Empty);
			}
			else
			{
				// Windows 7, 8 behave correctly
				Assert.That(exception.NativeErrorCode, Is.EqualTo(126));
				Assert.That(exception.Message, Is.Not.Empty);
			}
		}

	}
}

#endif
