﻿#region Copyright (c) 2013-05, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using System.Text;
using NUnit.Framework;


namespace Amarok
{
	/// <summary>
	/// </summary>
	[TestFixture]
	public class Test_CrcCalculator
	{
		[Test]
		[ExpectedException(typeof(ArgumentOutOfRangeException))]
		public void TestCrcCalculatorWithOrderBelowOne()
		{
			var calc = new CrcCalculator(0, 0x1021, false, 0xffff, 0x0000, false, false);
		}

		[Test]
		[ExpectedException(typeof(ArgumentOutOfRangeException))]
		public void TestCrcCalculatorWithOrderAbove32()
		{
			var calc = new CrcCalculator(33, 0x1021, false, 0xffff, 0x0000, false, false);
		}

		[Test]
		public void TestCrcCalculator()
		{
			var buffer = Encoding.ASCII.GetBytes("123456789");
			var calc = new CrcCalculator(16, 0x1021, false, 0xffff, 0x0000, false, false);
			Assert.AreEqual(0xE5CC, calc.Calculate(buffer, 0, buffer.Length));
		}

		[Test]
		public void TestCrcCalculatorWithReflectInput()
		{
			var buffer = Encoding.ASCII.GetBytes("123456789");
			var calc = new CrcCalculator(16, 0x1021, false, 0xffff, 0x0000, true, false);
			Assert.AreEqual(0x458B, calc.Calculate(buffer, 0, buffer.Length));
		}

		[Test]
		public void TestCrcCalculatorWithReflectOutput()
		{
			var buffer = Encoding.ASCII.GetBytes("123456789");
			var calc = new CrcCalculator(16, 0x1021, false, 0xffff, 0x0000, false, true);
			Assert.AreEqual(0x33A7, calc.Calculate(buffer, 0, buffer.Length));
		}

		[Test]
		public void TestCrcCalculatorWithDirect()
		{
			var buffer = Encoding.ASCII.GetBytes("123456789");
			var calc = new CrcCalculator(16, 0x1021, true, 0xffff, 0x0000, false, false);
			Assert.AreEqual(0x29B1, calc.Calculate(buffer, 0, buffer.Length));
		}

	}
}

#endif
