﻿#region Copyright (c) 2012-10, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using NUnit.Framework;


namespace Amarok
{
	[TestFixture]
	public class Test_TaskHelper
	{
		[Test]
		public void Test_Canceled()
		{
			var task1 = TaskHelper.Canceled;
			var task2 = TaskHelper.Canceled;

			Assert.That(task1, Is.Not.Null);
			Assert.That(task2, Is.SameAs(task1));

			Assert.That(task1.IsCompleted, Is.True);
			Assert.That(task1.IsCanceled, Is.True);
			Assert.That(task1.IsFaulted, Is.False);
		}

		[Test]
		public void Test_Completed()
		{
			var task1 = TaskHelper.Completed;
			var task2 = TaskHelper.Completed;

			Assert.That(task1, Is.Not.Null);
			Assert.That(task2, Is.SameAs(task1));

			Assert.That(task1.IsCompleted, Is.True);
			Assert.That(task1.IsCanceled, Is.False);
			Assert.That(task1.IsFaulted, Is.False);
		}


		[Test]
		public void Test_WhenAll_Enumerator()
		{
			IEnumerable<Task> tasks = new[] { 
				Task.Factory.StartNew(() => Thread.Sleep(50)),
				Task.Factory.StartNew(() => Thread.Sleep(50)),
				Task.Factory.StartNew(() => Thread.Sleep(50))
			};

			var task = TaskHelper.WhenAll(tasks);

			task.Wait();
			Assert.That(task.IsCompleted, Is.True);
		}

		[Test]
		public void Test_WhenAll_Params()
		{
			var task = TaskHelper.WhenAll(
				Task.Factory.StartNew(() => Thread.Sleep(50)),
				Task.Factory.StartNew(() => Thread.Sleep(50)),
				Task.Factory.StartNew(() => Thread.Sleep(50))
			);

			task.Wait();
			Assert.That(task.IsCompleted, Is.True);
		}

		[Test]
		public void Test_Run()
		{
			var task = TaskHelper.WhenAll(
				TaskHelper.Run(() => Thread.Sleep(50)),
				TaskHelper.Run(() => Thread.Sleep(50)),
				TaskHelper.Run(() => Thread.Sleep(50))
			);

			task.Wait();
			Assert.That(task.IsCompleted, Is.True);
		}

	}
}

#endif
