﻿#region Copyright (c) 09/19/2012, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using Amarok.Agents;
using NUnit.Framework;


namespace Amarok.Presentation
{
	[TestFixture]
	public class Test_ShowWindowAction
	{
		private class TestWindow : ViewWindow
		{
		}

		private class TestViewModel
		{
		}

		private class TestController : Controller<TestViewModel>
		{
			public TestController(AgentEnvironment environment, AgentOptions options, IView view, TestViewModel viewModel)
				: base(environment, options, view, viewModel)
			{
			}
		}

		private class TestWindowDescription : WindowDescription<
			TestWindow,
			TestController,
			TestViewModel>
		{
		}


		// --- TESTS ---


		[Test]
		public void Test_Usage()
		{
			var windowDesc = new TestWindowDescription();
			var environment = new AgentEnvironment();
			var options = new AgentOptions();

			var action = new ShowWindowAction(
				windowDesc,
				environment,
				options);

			Assert.That(action.WindowDescription, Is.SameAs(windowDesc));
			Assert.That(action.Environment, Is.SameAs(environment));
			Assert.That(action.Options, Is.SameAs(options));
		}

		[Test]
		public void Test_Construction_NullWindowDescription()
		{
			var environment = new AgentEnvironment();
			var options = new AgentOptions();

			Assert.Throws<ArgumentNullException>(() =>
				{
					var action = new ShowWindowAction(
						null,
						environment,
						options);
				});
		}

		[Test]
		public void Test_Construction_NullEnvironment()
		{
			var windowDesc = new TestWindowDescription();
			var options = new AgentOptions();

			Assert.Throws<ArgumentNullException>(() =>
				{
					var action = new ShowWindowAction(
						windowDesc,
						null,
						options);
				});
		}

		[Test]
		public void Test_Construction_NullOptions()
		{
			var windowDesc = new TestWindowDescription();
			var environment = new AgentEnvironment();

			Assert.Throws<ArgumentNullException>(() =>
				{
					var action = new ShowWindowAction(
						windowDesc,
						environment,
						null);
				});
		}

	}
}

#endif
