﻿#region Copyright (c) 2013-05, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using NUnit.Framework;


namespace Amarok.Presentation
{
	[TestFixture]
	public class Test_Controller
	{
		[Test]
		public void Test_Factory()
		{
			var factory1 = Controller.Factory;
			var factory2 = Controller.Factory;

			Assert.That(factory1, Is.TypeOf<DefaultControllerFactory>());
			Assert.That(factory2, Is.SameAs(factory1));
		}

		[Test]
		public void Test_SetFactory()
		{
			Assert.That(Controller.Factory, Is.TypeOf<DefaultControllerFactory>());

			var factory = new DummyControllerFactory();
			Controller.SetFactory(factory);

			Assert.That(Controller.Factory, Is.SameAs(factory));

			Controller.SetFactory(new DefaultControllerFactory());

			Assert.That(Controller.Factory, Is.TypeOf<DefaultControllerFactory>());
		}

		[Test]
		public void Test_SetFactory_Null()
		{
			Assert.That(Controller.Factory, Is.TypeOf<DefaultControllerFactory>());

			Assert.Throws<ArgumentNullException>(() =>
				{
					Controller.SetFactory(null);
				});

			Assert.That(Controller.Factory, Is.TypeOf<DefaultControllerFactory>());
		}

	}
}

#endif
