﻿#region Copyright (c) 07/17/2012, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System.Windows;
using NUnit.Framework;


namespace Amarok.Presentation
{
	[TestFixture]
	public class Test_BooleanToObjectConverter
	{
		[Test]
		public void Test_Convert_DefaultValues()
		{
			// :: Convert
			// True   ...  TrueValue property
			// False  ...  FalseValue property

			var converter = new BooleanToObjectConverter();

			Assert.That(converter.TrueValue, Is.True);
			Assert.That(converter.FalseValue, Is.False);
			Assert.That(converter.DefaultValue, Is.EqualTo(DependencyProperty.UnsetValue));

			Assert.That(
				converter.Convert(true, null, null, null),
				Is.EqualTo(true));

			Assert.That(
				converter.Convert(false, null, null, null),
				Is.EqualTo(false));

			Assert.That(
				converter.Convert(null, null, null, null),
				Is.EqualTo(null));

			Assert.That(
				converter.Convert("", null, null, null),
				Is.EqualTo(""));

			Assert.That(
				converter.Convert(123, null, null, null),
				Is.EqualTo(123));
		}

		[Test]
		public void Test_Convert_String()
		{
			// :: Convert
			// True   ...  TrueValue property
			// False  ...  FalseValue property

			var converter = new BooleanToObjectConverter()
			{
				TrueValue = "T",
				FalseValue = "F",
				DefaultValue = "D",
			};

			Assert.That(
				converter.Convert(true, null, null, null),
				Is.EqualTo("T"));

			Assert.That(
				converter.Convert(false, null, null, null),
				Is.EqualTo("F"));

			Assert.That(
				converter.Convert(null, null, null, null),
				Is.EqualTo(null));

			Assert.That(
				converter.Convert("", null, null, null),
				Is.EqualTo(""));

			Assert.That(
				converter.Convert(123, null, null, null),
				Is.EqualTo(123));
		}

		[Test]
		public void Test_Convert_Int32()
		{
			// :: Convert
			// True   ...  TrueValue property
			// False  ...  FalseValue property

			var converter = new BooleanToObjectConverter()
			{
				TrueValue = 111,
				FalseValue = 222,
				DefaultValue = 999,
			};

			Assert.That(
				converter.Convert(true, null, null, null),
				Is.EqualTo(111));

			Assert.That(
				converter.Convert(false, null, null, null),
				Is.EqualTo(222));

			Assert.That(
				converter.Convert(null, null, null, null),
				Is.EqualTo(null));

			Assert.That(
				converter.Convert("", null, null, null),
				Is.EqualTo(""));

			Assert.That(
				converter.Convert(123, null, null, null),
				Is.EqualTo(123));
		}


		[Test]
		public void Test_ConvertBack_DefaultValues()
		{
			// :: ConvertBack
			// equal to TrueValue   ...  True
			// equal to FalseValue  ...  False
			// Otherwise            ...  DefaultValue

			var converter = new BooleanToObjectConverter();

			Assert.That(
				converter.ConvertBack(true, null, null, null),
				Is.True);

			Assert.That(
				converter.ConvertBack(false, null, null, null),
				Is.False);

			Assert.That(
				converter.ConvertBack(null, null, null, null),
				Is.EqualTo(DependencyProperty.UnsetValue));

			Assert.That(
				converter.ConvertBack("", null, null, null),
				Is.EqualTo(DependencyProperty.UnsetValue));

			Assert.That(
				converter.ConvertBack(123, null, null, null),
				Is.EqualTo(DependencyProperty.UnsetValue));
		}

		[Test]
		public void Test_ConvertBack_String()
		{
			// :: ConvertBack
			// equal to TrueValue   ...  True
			// equal to FalseValue  ...  False
			// Otherwise            ...  DefaultValue

			var converter = new BooleanToObjectConverter()
			{
				TrueValue = "T",
				FalseValue = "F",
				DefaultValue = "D",
			};

			Assert.That(
				converter.ConvertBack("T", null, null, null),
				Is.True);

			Assert.That(
				converter.ConvertBack("F", null, null, null),
				Is.False);

			Assert.That(
				converter.ConvertBack(null, null, null, null),
				Is.EqualTo("D"));

			Assert.That(
				converter.ConvertBack("", null, null, null),
				Is.EqualTo("D"));

			Assert.That(
				converter.ConvertBack(123, null, null, null),
				Is.EqualTo("D"));
		}

		[Test]
		public void Test_ConvertBack_Int32()
		{
			// :: ConvertBack
			// equal to TrueValue   ...  True
			// equal to FalseValue  ...  False
			// Otherwise            ...  DefaultValue

			var converter = new BooleanToObjectConverter()
			{
				TrueValue = 111,
				FalseValue = 222,
				DefaultValue = 999,
			};

			Assert.That(
				converter.ConvertBack(111, null, null, null),
				Is.True);

			Assert.That(
				converter.ConvertBack(222, null, null, null),
				Is.False);

			Assert.That(
				converter.ConvertBack(null, null, null, null),
				Is.EqualTo(999));

			Assert.That(
				converter.ConvertBack("", null, null, null),
				Is.EqualTo(999));

			Assert.That(
				converter.ConvertBack(123, null, null, null),
				Is.EqualTo(999));
		}

	}
}

#endif
