﻿#region Copyright (c) 07/17/2012, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System.Windows;
using NUnit.Framework;


namespace Amarok.Presentation
{
	[TestFixture]
	public class Test_HiddenWhenNotNullOrEmptyConverter
	{
		[Test]
		public void Test_Convert()
		{
			// :: Convert
			// Null       ...  Visible
			// ""         ...  Visible
			// "  "       ...  Hidden
			// Otherwise  ...  Hidden

			Assert.That(
				HiddenWhenNotNullOrEmptyConverter.Instance.Convert(null, null, null, null),
				Is.EqualTo(Visibility.Visible));

			Assert.That(
				HiddenWhenNotNullOrEmptyConverter.Instance.Convert("", null, null, null),
				Is.EqualTo(Visibility.Visible));

			Assert.That(
				HiddenWhenNotNullOrEmptyConverter.Instance.Convert("  ", null, null, null),
				Is.EqualTo(Visibility.Hidden));

			Assert.That(
				HiddenWhenNotNullOrEmptyConverter.Instance.Convert("BLAH", null, null, null),
				Is.EqualTo(Visibility.Hidden));

			Assert.That(
				HiddenWhenNotNullOrEmptyConverter.Instance.Convert(123, null, null, null),
				Is.EqualTo(123));
		}

		[Test]
		public void Test_ConvertBack()
		{
			// :: ConvertBack
			// Always  ...  DependencyProperty.UnsetValue

			Assert.That(
				HiddenWhenNotNullOrEmptyConverter.Instance.ConvertBack(null, null, null, null),
				Is.EqualTo(DependencyProperty.UnsetValue));

			Assert.That(
				HiddenWhenNotNullOrEmptyConverter.Instance.ConvertBack("", null, null, null),
				Is.EqualTo(DependencyProperty.UnsetValue));

			Assert.That(
				HiddenWhenNotNullOrEmptyConverter.Instance.ConvertBack("  ", null, null, null),
				Is.EqualTo(DependencyProperty.UnsetValue));

			Assert.That(
				HiddenWhenNotNullOrEmptyConverter.Instance.ConvertBack("BLAH", null, null, null),
				Is.EqualTo(DependencyProperty.UnsetValue));

			Assert.That(
				HiddenWhenNotNullOrEmptyConverter.Instance.ConvertBack(123, null, null, null),
				Is.EqualTo(DependencyProperty.UnsetValue));
		}

	}
}

#endif
