﻿#region Copyright (c) 07/16/2012, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System.Windows;
using NUnit.Framework;


namespace Amarok.Presentation
{
	[TestFixture]
	public class Test_HiddenWhenTrueConverter
	{
		[Test]
		public void Test_Convert()
		{
			// :: Convert
			// True   ...  Hidden
			// False  ...  Visible

			Assert.That(
				HiddenWhenTrueConverter.Instance.Convert(true, null, null, null),
				Is.EqualTo(Visibility.Hidden));

			Assert.That(
				HiddenWhenTrueConverter.Instance.Convert(false, null, null, null),
				Is.EqualTo(Visibility.Visible));

			Assert.That(
				HiddenWhenTrueConverter.Instance.Convert(null, null, null, null),
				Is.EqualTo(null));

			Assert.That(
				HiddenWhenTrueConverter.Instance.Convert("", null, null, null),
				Is.EqualTo(""));

			Assert.That(
				HiddenWhenTrueConverter.Instance.Convert(123, null, null, null),
				Is.EqualTo(123));
		}

		[Test]
		public void Test_ConvertBack()
		{
			// :: ConvertBack
			// Visible    ...  False
			// Collapsed  ...  True
			// Hidden     ...  True

			Assert.That(
				HiddenWhenTrueConverter.Instance.ConvertBack(Visibility.Visible, null, null, null),
				Is.False);

			Assert.That(
				HiddenWhenTrueConverter.Instance.ConvertBack(Visibility.Hidden, null, null, null),
				Is.True);

			Assert.That(
				HiddenWhenTrueConverter.Instance.ConvertBack(Visibility.Collapsed, null, null, null),
				Is.True);

			Assert.That(
				HiddenWhenTrueConverter.Instance.ConvertBack(null, null, null, null),
				Is.EqualTo(null));

			Assert.That(
				HiddenWhenTrueConverter.Instance.ConvertBack("", null, null, null),
				Is.EqualTo(""));

			Assert.That(
				HiddenWhenTrueConverter.Instance.ConvertBack(123, null, null, null),
				Is.EqualTo(123));
		}

	}
}

#endif
