﻿#region Copyright (c) 11/27/2011, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using NUnit.Framework;


namespace Amarok.Presentation
{
	[TestFixture]
	public class Test_ViewActionHandler
	{
		private class MyViewAction : ViewAction
		{
		}


		// --- TESTS ---


		[Test]
		public void Test_Invoke()
		{
			MyViewAction callbackMessage = null;
			Int32 callbackCalled = 0;

			var callback = new Func<MyViewAction, Boolean>(msg =>
				{
					callbackMessage = msg;
					callbackCalled++;
					return true;
				});

			var handler = new ViewActionHandler<MyViewAction>(callback);
			var handlerInterface = (IHandler)handler;

			Assert.That(handler.Func, Is.SameAs(callback));

			var message = new MyViewAction();
			var handled = handlerInterface.Invoke(message);

			Assert.That(callbackCalled, Is.EqualTo(1));
			Assert.That(callbackMessage, Is.SameAs(message));
			Assert.That(handled, Is.True);
		}

		[Test]
		public void Test_Invoke_Exception()
		{
			MyViewAction callbackMessage = null;
			Int32 callbackCalled = 0;

			var callback = new Func<MyViewAction, Boolean>(msg =>
				{
					callbackMessage = msg;
					callbackCalled++;

					throw new ApplicationException("TEST");
				});

			var handler = new ViewActionHandler<MyViewAction>(callback);
			var handlerInterface = (IHandler)handler;

			Assert.That(handler.Func, Is.SameAs(callback));

			var message = new MyViewAction();
			var handled = handlerInterface.Invoke(message);

			Assert.That(callbackCalled, Is.EqualTo(1));
			Assert.That(callbackMessage, Is.SameAs(message));
			Assert.That(handled, Is.False);
		}

		[Test]
		public void Test_Construction_NullAction()
		{
			Assert.Throws<ArgumentNullException>(() =>
				{
					var handler = new ViewActionHandler<MyViewAction>(null);
				});
		}

	}
}

#endif
