﻿#region Copyright (c) 2013-05, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System.Windows.Controls;
using NUnit.Framework;


namespace Amarok.Presentation
{
	/// <summary>
	/// </summary>
	[TestFixture, RequiresSTA]
	public class Test_LocalizeHelper
	{
		[Test]
		public void Test_Localize_AccessText()
		{
			var control = new AccessText();

			Assert.That(control.Text, Is.Empty);
			Assert.That(control.ToolTip, Is.Null);

			control.Localize("A");

			Assert.That(control.Text, Is.EqualTo("A"));
			Assert.That(control.ToolTip, Is.Null);

			control.Localize("B", "C");

			Assert.That(control.Text, Is.EqualTo("B"));
			Assert.That(control.ToolTip, Is.EqualTo("C"));
		}

		[Test]
		public void Test_Localize_TextBlock()
		{
			var control = new TextBlock();

			Assert.That(control.Text, Is.Empty);
			Assert.That(control.ToolTip, Is.Null);

			control.Localize("A");

			Assert.That(control.Text, Is.EqualTo("A"));
			Assert.That(control.ToolTip, Is.Null);

			control.Localize("B", "C");

			Assert.That(control.Text, Is.EqualTo("B"));
			Assert.That(control.ToolTip, Is.EqualTo("C"));
		}

		[Test]
		public void Test_Localize_ContentControl()
		{
			var control = new ContentControl();

			Assert.That(control.Content, Is.Null);
			Assert.That(control.ToolTip, Is.Null);

			control.Localize("A");

			Assert.That(control.Content, Is.EqualTo("A"));
			Assert.That(control.ToolTip, Is.Null);

			control.Localize("B", "C");

			Assert.That(control.Content, Is.EqualTo("B"));
			Assert.That(control.ToolTip, Is.EqualTo("C"));
		}

		[Test]
		public void Test_Localize_ContentControl_Button()
		{
			var control = new Button();

			Assert.That(control.Content, Is.Null);
			Assert.That(control.ToolTip, Is.Null);

			control.Localize("A");

			Assert.That(control.Content, Is.EqualTo("A"));
			Assert.That(control.ToolTip, Is.Null);

			control.Localize("B", "C");

			Assert.That(control.Content, Is.EqualTo("B"));
			Assert.That(control.ToolTip, Is.EqualTo("C"));
		}

		[Test]
		public void Test_Localize_HeaderedContentControl()
		{
			var control = new HeaderedContentControl();

			Assert.That(control.Header, Is.Null);
			Assert.That(control.Content, Is.Null);
			Assert.That(control.ToolTip, Is.Null);

			control.Localize("A");

			Assert.That(control.Header, Is.EqualTo("A"));
			Assert.That(control.Content, Is.Null);
			Assert.That(control.ToolTip, Is.Null);

			control.Localize("B", "C");

			Assert.That(control.Header, Is.EqualTo("B"));
			Assert.That(control.Content, Is.Null);
			Assert.That(control.ToolTip, Is.EqualTo("C"));
		}

		[Test]
		public void Test_Localize_HeaderedContentControl_Expander()
		{
			var control = new Expander();

			Assert.That(control.Header, Is.Null);
			Assert.That(control.Content, Is.Null);
			Assert.That(control.ToolTip, Is.Null);

			control.Localize("A");

			Assert.That(control.Header, Is.EqualTo("A"));
			Assert.That(control.Content, Is.Null);
			Assert.That(control.ToolTip, Is.Null);

			control.Localize("B", "C");

			Assert.That(control.Header, Is.EqualTo("B"));
			Assert.That(control.Content, Is.Null);
			Assert.That(control.ToolTip, Is.EqualTo("C"));
		}


		[Test]
		public void Test_Localize_HeaderedItemsControl_MenuItem()
		{
			var control = new MenuItem();

			Assert.That(control.Header, Is.Null);
			Assert.That(control.ToolTip, Is.Null);

			control.Localize("A");

			Assert.That(control.Header, Is.EqualTo("A"));
			Assert.That(control.ToolTip, Is.Null);

			control.Localize("B", "C");

			Assert.That(control.Header, Is.EqualTo("B"));
			Assert.That(control.ToolTip, Is.EqualTo("C"));
		}

	}
}

#endif
