﻿#region Copyright (c) 2013-05, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using NUnit.Framework;


namespace Amarok.Presentation
{
	/// <summary>
	/// </summary>
	[TestFixture]
	public class Test_Wpf
	{
		[Test]
		public void Test_ToPixel()
		{
			// 1 in = 96 px
			Assert.That(Wpf.InchToPixel(0.0), Is.EqualTo(0.0));
			Assert.That(Wpf.InchToPixel(0.5), Is.EqualTo(48.0));
			Assert.That(Wpf.InchToPixel(1.0), Is.EqualTo(96.0));

			// 1 in = 25.4 mm
			Assert.That(Wpf.MillimeterToPixel(0.0), Is.EqualTo(0.0));
			Assert.That(Wpf.MillimeterToPixel(12.7), Is.EqualTo(48.0).Within(0.00001));
			Assert.That(Wpf.MillimeterToPixel(25.4), Is.EqualTo(96.0).Within(0.00001));
		}

		[Test]
		public void Test_ToMillimeter()
		{
			// 1 in = 25.4 mm
			Assert.That(Wpf.InchToMillimeter(0.0), Is.EqualTo(0.0));
			Assert.That(Wpf.InchToMillimeter(0.5), Is.EqualTo(12.7));
			Assert.That(Wpf.InchToMillimeter(1.0), Is.EqualTo(25.4));

			// 1 in = 96 px
			Assert.That(Wpf.PixelToMillimeter(0.0), Is.EqualTo(0.0));
			Assert.That(Wpf.PixelToMillimeter(48.0), Is.EqualTo(12.7).Within(0.00001));
			Assert.That(Wpf.PixelToMillimeter(96.0), Is.EqualTo(25.4).Within(0.00001));
		}

		[Test]
		public void Test_ToInch()
		{
			// 1 in = 96 px
			Assert.That(Wpf.PixelToInch(0.0), Is.EqualTo(0.0));
			Assert.That(Wpf.PixelToInch(48.0), Is.EqualTo(0.5));
			Assert.That(Wpf.PixelToInch(96.0), Is.EqualTo(1.0));

			// 1 in = 25.4 mm
			Assert.That(Wpf.MillimeterToInch(0.0), Is.EqualTo(0.0));
			Assert.That(Wpf.MillimeterToInch(12.7), Is.EqualTo(0.5));
			Assert.That(Wpf.MillimeterToInch(25.4), Is.EqualTo(1.0));
		}

		[Test]
		public void Test_FontSize()
		{
			Assert.That(Wpf.FontSizePointToPixel(0.0), Is.EqualTo(0.0));
			Assert.That(Wpf.FontSizePointToPixel(6.0), Is.EqualTo(8.0));
			Assert.That(Wpf.FontSizePointToPixel(12.0), Is.EqualTo(16.0));

			Assert.That(Wpf.FontSizePixelToPoint(0.0), Is.EqualTo(0.0));
			Assert.That(Wpf.FontSizePixelToPoint(8.0), Is.EqualTo(6.0));
			Assert.That(Wpf.FontSizePixelToPoint(16.0), Is.EqualTo(12.0));
		}

	}
}

#endif
