﻿#region Copyright (c) 2013-02, Olaf Kober <amarok.projects@gmail.com>
//================================================================================
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.
//================================================================================
#endregion

#if TESTS

using System;
using NUnit.Framework;


namespace Amarok.Presentation
{
	[TestFixture]
	public class Test_ViewModel_T
	{
		private class MyRootViewModel : ViewModel
		{
		}

		private class MyViewModel : ViewModel<MyRootViewModel>
		{
			public MyViewModel()
			{
			}

			public MyViewModel(MyRootViewModel parent)
				: base(parent)
			{
			}
		}


		// --- TESTS ---


		[Test]
		public void Test_Parent()
		{
			var root = new MyRootViewModel();
			var child = new MyViewModel(root);
			var child2 = new MyViewModel();

			Assert.That(root.HasParent, Is.False);
			Assert.That(root.Parent, Is.Null);
			Assert.That(child.HasParent, Is.True);
			Assert.That(child.Parent, Is.SameAs(root));
			Assert.That(child2.HasParent, Is.False);
			Assert.That(child2.Parent, Is.Null);

			child.BeginRecording();
			child.Parent = null;

			Assert.That(child.GetRecording().AsString(";"), Is.EqualTo("Parent;HasParent"));
			Assert.That(child.HasParent, Is.False);
			Assert.That(child.Parent, Is.Null);

			child.BeginRecording();
			child.Parent = root;

			Assert.That(child.GetRecording().AsString(";"), Is.EqualTo("Parent;HasParent"));
			Assert.That(child.HasParent, Is.True);
			Assert.That(child.Parent, Is.SameAs(root));
		}

		[Test]
		public void Test_Parent_Null()
		{
			Assert.Throws<ArgumentNullException>(() =>
				{
					var child = new MyViewModel(null);
				});
		}

	}
}

#endif
